import Option, { None, Some } from '../option.ts';
import {
	AbstractFileType,
	defaultHighlightOptions,
	FileLang,
	HighlightingOptions,
} from './base.ts';

export class CSSFile extends AbstractFileType {
	public override readonly name: FileLang = FileLang.CSS;
	public override readonly singleLineComment = None;
	public override readonly multiLineCommentStart: Option<string> = Some('/*');
	public override readonly multiLineCommentEnd: Option<string> = Some('*/');
	public override readonly keywords1 = [
		':active',
		':any-link',
		':autofill',
		':checked',
		':default',
		':disabled',
		':empty',
		':enabled',
		':first-child',
		':first-of-type',
		':focus-visible',
		':focus-within',
		':focus',
		':fullscreen',
		':hover',
		':in-range',
		':indeterminate',
		':invalid',
		':last-child',
		':last-of-type',
		':link',
		':modal',
		':nth-child',
		':nth-last-child',
		':nth-last-of-type',
		':nth-of-type',
		':only-child',
		':only-of-type',
		':optional',
		':out-of-range',
		':paused',
		':picture-in-picture',
		':placeholder-shown',
		':playing',
		':read-only',
		':read-write',
		':required',
		':root',
		':scope',
		':target',
		':user-valid',
		':valid',
		':visited',
		'::after',
		'::backdrop',
		'::before',
		'::cue',
		'::file-selector-button',
		'::first-letter',
		'::first-line',
		'::grammar-error',
		'::marker',
		'::placeholder',
		'::selection',
		'::spelling-error',
		'@charset',
		'@color-profile',
		'@container',
		'@counter-style',
		'@font-face',
		'@font-feature-values',
		'@font-palette-values',
		'@import',
		'@keyframes',
		'@layer',
		'@media',
		'@namespace',
		'@page',
		'@position-try',
		'@property',
		'@scope',
		'@starting-style',
		'@supports',
		'@view-transition',
	];
	public override readonly keywords2 = [
		'contain-intrinsic-inline-size',
		'contain-intrinsic-block-size',
		'scroll-padding-inline-start',
		'border-bottom-right-radius',
		'overscroll-behavior-inline',
		'scroll-margin-inline-start',
		'scroll-padding-block-start',
		'transition-timing-function',
		'animation-iteration-count',
		'animation-timing-function',
		'border-bottom-left-radius',
		'border-inline-start-color',
		'border-inline-start-style',
		'border-inline-start-width',
		'border-start-start-radius',
		'font-synthesis-small-caps',
		'overscroll-behavior-block',
		'repeating-linear-gradient',
		'repeating-radial-gradient',
		'scroll-margin-block-start',
		'scroll-padding-inline-end',
		'text-decoration-thickness',
		'border-block-start-color',
		'border-block-start-style',
		'border-block-start-width',
		'contain-intrinsic-height',
		'repeating-conic-gradient',
		'scroll-margin-inline-end',
		'scroll-padding-block-end',
		'text-decoration-skip-ink',
		'border-end-start-radius',
		'border-inline-end-color',
		'border-inline-end-style',
		'border-inline-end-width',
		'border-start-end-radius',
		'border-top-right-radius',
		'contain-intrinsic-width',
		'font-synthesis-position',
		'font-variant-alternates',
		'font-variant-east-asian',
		'font-variation-settings',
		'scroll-margin-block-end',
		'text-underline-position',
		'border-block-end-color',
		'border-block-end-style',
		'border-block-end-width',
		'border-top-left-radius',
		'font-language-override',
		'font-variant-ligatures',
		'position-try-fallbacks',
		'text-emphasis-position',
		'animation-composition',
		'animation-range-start',
		'background-attachment',
		'background-blend-mode',
		'background-position-x',
		'background-position-y',
		'border-end-end-radius',
		'font-feature-settings',
		'font-synthesis-weight',
		'font-variant-position',
		'grid-template-columns',
		'hyphenate-limit-chars',
		'overscroll-behavior-x',
		'overscroll-behavior-y',
		'scroll-padding-bottom',
		'scroll-padding-inline',
		'shape-image-threshold',
		'text-decoration-color',
		'text-decoration-style',
		'text-underline-offset',
		'animation-play-state',
		'box-decoration-break',
		'font-synthesis-style',
		'font-variant-numeric',
		'overflow-clip-margin',
		'padding-inline-start',
		'scroll-margin-bottom',
		'scroll-margin-inline',
		'scroll-padding-block',
		'scroll-padding-right',
		'scroll-timeline-axis',
		'scroll-timeline-name',
		'text-combine-upright',
		'text-decoration-line',
		'text-decoration-skip',
		'view-transition-name',
		'white-space-collapse',
		'animation-direction',
		'animation-fill-mode',
		'animation-range-end',
		'backface-visibility',
		'background-position',
		'border-bottom-color',
		'border-bottom-style',
		'border-bottom-width',
		'border-image-outset',
		'border-image-repeat',
		'border-image-source',
		'border-inline-color',
		'border-inline-start',
		'border-inline-style',
		'border-inline-width',
		'font-optical-sizing',
		'forced-color-adjust',
		'grid-template-areas',
		'hanging-punctuation',
		'hyphenate-character',
		'list-style-position',
		'margin-inline-start',
		'overscroll-behavior',
		'padding-block-start',
		'position-visibility',
		'scroll-margin-block',
		'scroll-margin-right',
		'scroll-padding-left',
		'text-emphasis-color',
		'text-emphasis-style',
		'transition-behavior',
		'transition-duration',
		'transition-property',
		'view-timeline-inset',
		'animation-duration',
		'animation-timeline',
		'border-block-color',
		'border-block-start',
		'border-block-style',
		'border-block-width',
		'border-image-slice',
		'border-image-width',
		'border-right-color',
		'border-right-style',
		'border-right-width',
		'content-visibility',
		'font-variant-emoji',
		'grid-template-rows',
		'margin-block-start',
		'mask-border-outset',
		'mask-border-repeat',
		'mask-border-source',
		'padding-inline-end',
		'perspective-origin',
		'position-try-order',
		'print-color-adjust',
		'scroll-margin-left',
		'scroll-padding-top',
		'view-timeline-axis',
		'view-timeline-name',
		'background-origin',
		'background-repeat',
		'border-inline-end',
		'border-left-color',
		'border-left-style',
		'border-left-width',
		'column-rule-color',
		'column-rule-style',
		'column-rule-width',
		'counter-increment',
		'font-variant-caps',
		'grid-auto-columns',
		'grid-column-start',
		'image-orientation',
		'inset-block-start',
		'margin-inline-end',
		'mask-border-slice',
		'mask-border-width',
		'masonry-auto-flow',
		'padding-block-end',
		'page-break-before',
		'scroll-margin-top',
		'scroll-snap-align',
		'stroke-dashoffset',
		'stroke-miterlimit',
		'text-spacing-trim',
		'background-color',
		'background-image',
		'border-block-end',
		'border-top-color',
		'border-top-style',
		'border-top-width',
		'font-size-adjust',
		'image-resolution',
		'inset-inline-end',
		'line-height-step',
		'list-style-image',
		'margin-block-end',
		'mask-border-mode',
		'page-break-after',
		'page-brak-inside',
		'page-orientation',
		'scroll-snap-stop',
		'scroll-snap-type',
		'scrollbar-gutter',
		'stroke-dasharray',
		'text-orientation',
		'text-size-adjust',
		'transform-origin',
		'transition-delay',
		'animation-delay',
		'backdrop-filter',
		'background-clip',
		'background-size',
		'border-collapse',
		'grid-column-end',
		'image-rendering',
		'inset-block-end',
		'justify-content',
		'list-style-type',
		'max-inline-size',
		'min-inline-size',
		'object-position',
		'offset-distance',
		'offset-position',
		'overflow-anchor',
		'overflow-inline',
		'override-colors',
		'position-anchor',
		'radial-gradient',
		'scroll-behavior',
		'scroll-timeline',
		'scrollbar-color',
		'scrollbar-width',
		'shape-rendering',
		'stroke-linejoin',
		'text-align-last',
		'text-decoration',
		'text-wrap-style',
		'transform-style',
		'animation-name',
		'animation-name',
		'border-spacing',
		'container-name',
		'container-type',
		'flex-direction',
		'font-synthesis',
		'grid-auto-flow',
		'grid-auto-rows',
		'grid-row-start',
		'initial-letter',
		'justify-tracks',
		'letter-spacing',
		'mask-composite',
		'min-block-size',
		'mix-blend-mode',
		'outline-offset',
		'overflow-block',
		'padding-bottom',
		'padding-inline',
		'pointer-events',
		'scroll-padding',
		'stroke-linecap',
		'stroke-opacity',
		'text-rendering',
		'text-transform',
		'text-wrap-mode',
		'timeline-scope',
		'vertical-align',
		'align-content',
		'border-bottom',
		'border-inline',
		'border-radius',
		'counter-reset',
		'grid-template',
		'justify-items',
		'margin-bottom',
		'margin-inline',
		'mask-position',
		'offset-anchor',
		'offset-rotate',
		'outline-color',
		'outline-style',
		'outline-width',
		'overflow-wrap',
		'padding-block',
		'padding-right',
		'place-content',
		'position-area',
		'ruby-position',
		'scroll-margin',
		'shape-outside',
		'text-emphasis',
		'text-overflow',
		'transform-box',
		'unicode-range',
		'vector-effect',
		'view-timeline',
		'accent-color',
		'align-tracks',
		'aspect-ratio',
		'border-block',
		'border-color',
		'border-image',
		'border-right',
		'border-style',
		'border-width',
		'break-before',
		'break-inside',
		'caption-side',
		'color-scheme',
		'column-count',
		'column-width',
		'field-sizing',
		'font-kerning',
		'font-palette',
		'font-stretch',
		'font-variant',
		'grid-row-end',
		'inset-inline',
		'justify-self',
		'margin-block',
		'margin-right',
		'padding-left',
		'position-try',
		'shape-margin',
		'stop-opacity',
		'stroke-width',
		'table-layout',
		'text-justify',
		'text-shaddow',
		'touch-action',
		'unicode-bidi',
		'word-spacing',
		'writing-mode',
		'align-items',
		'anchor-name',
		'border-left',
		'break-after',
		'caret-color',
		'column-fill',
		'column-rule',
		'column-span',
		'counter-set',
		'empty-cells',
		'flex-shrink',
		'font-family',
		'font-weight',
		'grid-column',
		'inline-size',
		'inset-block',
		'line-height',
		'margin-left',
		'margin-trim',
		'mask-border',
		'mask-origin',
		'mask-repeat',
		'offset-path',
		'paddint-top',
		'paint-order',
		'perspective',
		'place-items',
		'size-adjust',
		'text-anchor',
		'text-indent',
		'translate3d',
		'user-select',
		'white-space',
		'will-change',
		'align-self',
		'appearance',
		'background',
		'block-size',
		'border-top',
		'box-shadow',
		'box-sizing',
		'column-gap',
		'flex-basis',
		'font-style',
		'inset-area',
		'line-break',
		'line-clamp',
		'list-style',
		'margin-top',
		'mask-image',
		'math-depth',
		'math-shift',
		'math-style',
		'max-height',
		'min-height',
		'object-fit',
		'overflow-x',
		'overflow-y',
		'place-self',
		'ruby-align',
		'stop-color',
		'text-align',
		'transition',
		'translateX',
		'translateY',
		'translateZ',
		'visibility',
		'word-break',
		'animation',
		'clip-path',
		'container',
		'direction',
		'flex-flow',
		'flex-grow',
		'flex-wrap',
		'font-size',
		'grid-area',
		'isolation',
		'mask-clip',
		'mask-mode',
		'mask-size',
		'mask-type',
		'max-width',
		'min-width',
		'text-wrap',
		'transform',
		'translate',
		'word-wrap',
		'work-wrap',
		'grid-row',
		'position',
		'rotate3d',
		'saturate',
		'speak-as',
		'tab-size',
		'columns',
		'contain',
		'content',
		'display',
		'hyphens',
		'inherit',
		'initial',
		'opacity',
		'orphans',
		'outline',
		'overlay',
		'padding',
		'polygon',
		'rotateX',
		'rotateY',
		'rotateZ',
		'row-gap',
		'scale3d',
		'symbols',
		'z-index',
		'border',
		'bottom',
		'cursor',
		'filter',
		'height',
		'margin',
		'offset',
		'quotes',
		'repeat',
		'resize',
		'revert',
		'rotate',
		'scaleX',
		'scaleY',
		'scaleZ',
		'scroll',
		'stroke',
		'suffix',
		'syntax',
		'system',
		'widows',
		'clear',
		'color',
		'float',
		'inset',
		'order',
		'paint',
		'range',
		'right',
		'round',
		'scale',
		'sepia',
		'skewX',
		'skewY',
		'steps',
		'unset',
		'width',
		'width',
		'clip',
		'flex',
		'font',
		'grid',
		'left',
		'mask',
		'page',
		'path',
		'rect',
		'sign',
		'size',
		'skew',
		'sqrt',
		'view',
		'zoom',
		'all',
		'gap',
		'pow',
		'ray',
		'rem',
		'rgb',
		'sin',
		'src',
		'tan',
		'top',
		'url',
		'var',
		'rx',
		'ry',
		'r',
		'x',
		'y',
	];
	public override readonly operators = ['::', ':', ',', '+', '>', '~', '-'];
	public override readonly hlOptions: HighlightingOptions = {
		...defaultHighlightOptions,
	};
}
